﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;

namespace SilverArcade.SilverSprite.Graphics
{
    internal class SpriteList : List<Sprite>
    {
        SpriteBatchCanvas _spriteBatchCanvas;
        public Texture2D Texture2D;
        public SpriteFont SpriteFont;
        public Canvas ParentCanvas;
        public SpriteType Type
        {
            get;
            set;
        }

        public enum SpriteType
        {
            SpriteImage,
            SpriteText,
            ClippedSpriteImage
        }

        public SpriteList(SpriteBatchCanvas spriteBatchCanvas, SpriteType type)
            : base()
        {
            _spriteBatchCanvas = spriteBatchCanvas;
            Type = type;
        }

        int current;
        public void BeginSpriteBatch()
        {
            foreach (Sprite d in this)
            {
                d.InUse = false;
            }
            current = 0;
        }

        Sprite CreateSprite()
        {
            if (Type == SpriteType.SpriteText)
            {
                SpriteText s = new SpriteText();
                s.Font = SpriteFont;
                return s;
            }
            else if (Type == SpriteType.ClippedSpriteImage)
            {
                ClippedSpriteImage s = new ClippedSpriteImage();
                s.Texture2D = Texture2D;
                return (Sprite)s;
            }
            return null;
        }

        public Sprite GetSprite()
        {
            if (current < Count - 1)
            {
                Sprite d = this[current];
                current++;
                return d;
            }
            else
            {
                Sprite d = CreateSprite();
                this.Add(d);
                return d;
            }          
        }

        public void EndSpriteBatch()
        {
            foreach (Sprite d in this)
            {
                if (d.InUse == false && d.Active == true)
                {
                    d.Active = false;
                    ParentCanvas.Children.Remove(d.Element);
                }
                else if (d.InUse == true && d.Active == false)
                {
                    d.Active = true;
                    ParentCanvas.Children.Add(d.Element);
                }
            }
        }
    }
}
